# -----------------------------------------------------------------------------
#                       GRTOOL - A Link To Grid Interface
# -----------------------------------------------------------------------------
# FILE    : GRANNOTA.AWK  Version 2.5 
# AUTHOR  : Lukas Spiess, CH-2558 Aegerten
# DATE    :  6 November 1992
# LAST REV: 10 February 1994
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# REMARKS : - This utility generates EMME/2 annotation files. A parameter-file
#             and an input-file are recomended. (See grlinka and grparama)
#           - This file can be converted to an EXE file using AWKC:
#             awkc -o grannota.exe -f grannota.awk
#           - The calling syntax is:
#
#             grannota dn=data pf=parameter-file input-file [ >output-file ]
#
#             where grannota is either: awk -f grannota.awk or grannota.exe
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#                                    CHANGES   
# 06-04-92: - grid box drawing FOR loops substituted  
# 06-04-92: - printf: %12.3f --> %.3f  
# 06-11-92: - some bugs fixed
# 09-12-93: - File handling changed for compilation with AWKC so
#             the parameter-file must be renamed to paramet.par
# 16-12-93: - Some messages like processing filenames and number of lines
#             processed are added. 
#           - File handling changed once more parameter-file must be specified
#             by the variable pf=. The calling syntax is now:
#
#             grannota dn=data pf=parameter-file input-file [ >output-file ]
#
# 17-12-93: - File header revised 
# 27-01-94: - Filename changed to GRANNOTA.AWK
# 31-01-94: - Enhanced data selection procedure: You can now select the data
#             column either 
#             -- by specifying a data column NUMBER (E.g. 1=1.column...) or 
#             -- (NEW) by specifying a data NAME (E.g. @NOx, @O3)
#             In the latter case the input-file needs a comment line containing
#             the data NAMEs of all columns (labels) in the correct order.
#             This will be the case if the input-file has been generated with
#             grmacro and grlink.
#             If the specified NAME doesn't match any labels, the program 
#             prints a warning and exits.
#           - Data selection with variable dn=data (number or name)
#             overwrites the selection in the parameter-file
#             (Keyword DN data E.g. DN @NOx).
# 08-02-94: - Support for GDX and GDY instead of GD.
#           - LogFile changed to permanent Logfile grtool.log 
#           - Label line recognition adapted and comparison of GDX and
#             GDY in the label line (xg=GDX yg=GDY) with the specification
#             in the parameter-file (GD gdx gdy)
#           - Class definition structure adapted
# 10-02-94: - The same awk file can used on DOS and UNIX systmes without any
#             changes. The temporary file grtool.tmp, which has to be
#             created by the DOS batch file or by the UNIX shell script
#             allows to specify the message output device by the keyword
#             OUTPUT device and the sort program to be used is referenced by
#             the keyword GRSORT sort-program. Under this is usually 
#             GRSORT gsort and OUTPUT con, under UNIX GRSORT sort and
#             OUTPUT grtool.log or /dev/tty.
# ---------------------------------------------------------------------------

BEGIN {
    LogFile="grtool.log";
    MsgOutput=LogFile;
    while ((getline < "grtool.tmp") > 0)
      if ( $1 == "OUTPUT" ) MsgOutput=$2;

    xl=yl=xh=yh=LW=LH=NS=DT=DF=NX=NY=Title=xdisp=ydisp=tx=ty="*";
    xmin = ymin = wmin =  100000;
    xmax = ymax = wmax = -100000;
    lastColor= -1;
    l= 0;
    MaxCol= 10;
    data = 0;
    } 

NR == 1 {
    ParFile = pf;
    # Saving $0 (the first input-file line)
    rec = $0;
    while ((getline < ParFile) > 0) {
        l++;
        if ( $1 == "#") continue;
        if ( $1 == "CG" ) { CG     = $2; continue; }
        if ( $1 == "RP" ) { RP     = $2; continue; }
        if ( $1 == "LG" ) { LG     = $2; continue; }
        if ( $1 == "TS" ) { TS     = $2; continue; }
        if ( $1 == "TI" ) { TI     = $2; continue; }
        if ( $1 == "NC" ) { NC     = $2; continue; }
        if ( $1 == "DT" ) { DT     = $2; continue; }
        if ( $1 == "DF" ) { DF     = $2; continue; }
        if ( $1 == "DN" ) { Data   = $2; continue; }
        if ( $1 == "SC" ) { Scale  = $2; continue; }
        if ( $1 == "GC" ) { Color1 = $2; continue; }
        if ( $1 == "PC" ) { Color2 = $2; continue; }
        if ( $1 == "ND" ) { ND     = $2; continue; }
        if ( $1 == "NS" ) { NS     = $2; continue; }
        if ( $1 == "GD" ) { GDX    = $2; GDY  = $3; continue; }
        if ( $1 == "GO" ) { GOX    = $2; GOY  = $3; continue; }
        if ( $1 == "DG" ) { NX     = $2; NY   = $3; continue; }
        if ( $1 == "LK" ) { xl     = $2; yl   = $3; continue; }
        if ( $1 == "TK" ) { xh     = $2; yh   = $3; continue; }
        if ( $1 == "TD" ) { tx     = $2; ty   = $3; continue; }
        if ( $1 == "LB" ) { LW     = $2; LH   = $3; continue; }
        if ( $1 == "LD" ) { xdisp  = $2; ydisp= $3; continue; }
        if ( $1 == "LT" ) { 
            split($0,array,"'");
            Title = array[2]; 
            continue;
	    }
        if ( $1 == "LC" ) {
            i      = $2;
            lo[i]  = $3; hi[i]  = $4;
            c[i]   = $5;
            Hor[i] = Ver[i] = 0;
            if ( $6 != 0 ) {
              Hor[i] = 1; cdy[i] = $6;
              }
            if ( $7 != 0 ) {
              Ver[i] = 1; cdx[i] = $7;
              }
            split($0,array,"'");
            Text[i]= array[2]; 
            continue;
            }
        printf("FILE %s, LINE %d NOT RECOGNIZED:\n%s\n",ParFile,l,$0) >> MsgOutput;
        }
    printf("Parameter-file: %-12s (%4d lines read)\n", ParFile, l) >> MsgOutput;
    init();
    if(!GDY)GDY=GDX;
    print "u1 1";
    if ( RP == 1 ) {
        if ( DT == "*" ) DT = 5;
        if ( DF == "*" ) DF = 0;
        if ( NX == "*" ) NX = GDX/10;
        if ( NY == "*" ) NY = GDY/10;
        if ( NS == "*" ) NS = GDY/8;
        Precision = "'%" DT "." DF "f'";
        print "c" Color2;
        print "s" NS;
        }
    if ( RP == 2 )
        print "c" Color2;
    InFile = FILENAME;
    
    # Restoring $0 of the first line of the input file
    $0 = rec;
    }

$1 ~ /^xg/ {
  split($1,a,"=");
  if ( GDX != a[2] ) {
    print "Different GDX in input-file (" a[2] ") and parameter-file (" GDX ")" > MsgOutput;
  } else
    GDX = a[2];
  split($2,a,"=");
  if ( GDY != a[2] ) {
    print "Different GDY in input-file (" a[2] ") and parameter-file (" GDY ")" > MsgOutput;
  } else
    GDY = a[2];
  if (!data)
    for (i=3; i<=NF; i++) {
      if ( $i == Data) {
        p = i; data = 1;
        }
      } 
  print "Data selection: " $p " (DN= " Data ")" >> MsgOutput;
  next;
  }


!/^#/ {
    if (!data) {
       print "ERROR: Data selection failed. (DN= " Data ")" >> MsgOutput;
       exit(1);
       }
    if      ( op == 0 ) 
        Wert = $p;
    else if ( op == 1 ) 
        Wert = $p*$q;
    else if ( op == 2 ) 
        Wert = $p/$q;
    else if ( op == 3 ) 
        Wert = $p+$q;
    else
        Wert = $p-$q;
# Maximum and minimum coordinates and value
    xmin>$1 ? xmin=$1 : 0; ymin>$2 ? ymin=$2 : 0; wmin>Wert ? wmin=Wert : 0;
    xmax<$1 ? xmax=$1 : 0; ymax<$2 ? ymax=$2 : 0; wmax<Wert ? wmax=Wert : 0;
# Class Representation
    if ( RP == 0 ) {
        for (i=1; i <= NC; i++) {
            if ( (Wert-lo[i]) >= 0 && (Wert-hi[i]) < 0) {
                if ( lastColor != c[i] )
                    printf("c%d\n",(lastColor = c[i]));           
                if ( Ver[i] == 1 ) {
                    dx = cdx[i];
                    if ( (step = int(($1-GOX)/dx)*dx+GOX) <= $1 ) step += dx;
                    for (t=step; t < $1+GDX; t+=dx) {
                        printf("m%.3f %.3f\n",t,$2);
                        printf("d%.3f %.3f\n",t,$2+GDY);
                        }
                    }
                if ( Hor[i] == 1 ) {
                    dy = cdy[i];
                    if ( (step = int(($2-GOY)/dy)*dy+GOY) <= $2 ) step += dy;
                    for (t=step; t < $2+GDY; t+=dy) {
                        printf("m%.3f %.3f\n",$1,t);
                        printf("d%.3f %.3f\n",$1+GDX,t);
                        }
                    }
                }
            }
        next;
        }
# Number Representation
    if ( RP == 1 ) {
        printf("t%.3f %.3f " Precision "\n",$1+NX,$2+NY,Wert);
        next;
        }
# Proportional Representation
    if ( RP == 2 ) {
        if (Wert!=0) {
            step = GDX/(int(Wert+Scale/2)/Scale+2);
        } else
            step = GDX;
        for (t=$1+step; t < $1+GDX-step/3; t+=step) {
            printf("m%.3f %.3f\n",t,$2);
            printf("d%.3f %.3f\n",t,$2+GDY);
            }
        next;
        }
    }

function Legend() {
    if ( xl    == "*" ) xl    = 0;
    if ( yl    == "*" ) yl    = 0;
    if ( LW    == "*" ) LW    = GDX;
    if ( LH    == "*" ) LH    = GDY;
    if ( xdisp == "*" ) xdisp = 0;
    if ( ydisp == "*" ) ydisp =-LH;
    if ( TS    == 0   ) TS    = LH*0.25;
    if ( TI    == 0   ) TI    = LH*0.5;
    if ( tx    == "*" ) tx    = LW+TS;
    if ( ty    == "*" ) ty    = (LH-TS)/2;
    if ( xh    == "*" ) xh    = 0;
    if ( yh    == "*" ) yh    = LH+TS;
    if ( Title == "*" ) Title = "LEGEND";
    x1 = xl;
    y1 = yl;
    print "s" TS;
    printf("c%d\n",Color1);           
    for (k=1; k <= NC; k++) {
        printf("c%d\n",c[k]);           
        if ( Ver[k] == 1 ) {
            dx = cdx[k];
            if ( (step = int((xl-GOX)/dx)*dx+GOX) < xl) step += dx;
            for (t=step; t < xl+LW; t+=dx) {
                printf("m%.3f %.3f\n",t,yl);
                printf("d%.3f %.3f\n",t,yl+LH);
                }
            }
        if ( Hor[k] == 1 ) {
            dy = cdy[k];
            if ( (step = int((yl-GOY)/dy)*dy+GOY) < yl ) step += dy;
            for (t=step; t < yl+LH; t+=dy) {
                printf("m%.3f %.3f\n",xl,t);
                printf("d%.3f %.3f\n",xl+LW,t);
                }
            }
        printf("t%.3f %.3f '%s'\n",xl+tx,yl+ty,Text[k]);
        print "c" Color1;
        for (i=xl; i <= xl+LW; i+=LW) {
            printf("m%.3f %.3f\n",i,yl);
            printf("d%.3f %.3f\n",i,yl+LH);
            }
        for (i=yl; i <= yl+LH; i+=LH) {
            printf("m%.3f %.3f\n",xl,i);
            printf("d%.3f %.3f\n",xl+LW,i);
            }
	xl += xdisp;
	yl += ydisp;
        }
    print "s" TI;
    printf("t%.3f %.3f '%s'\n",x1+xh,y1+yh,Title);
    }

function init() {
  # Redefinition of data selection by commandline variable  dn=data
  if (dn)
      Data = dn;
  if ( Data ~ /^[1-9]+[0-9]*/ ) {
    op = 0;
    p  = q = 2;
    if      ( (pos = index(Data,"*")) )
        op  = 1;
    else if ( (pos = index(Data,"/")) )
        op  = 2; 
    else if ( (pos = index(Data,"+")) )
        op  = 3;
    else if ( (pos = index(Data,"-")) )
        op  = 4;
    if ( op ) {
        p += substr(Data, 1, pos-1);
        q += substr(Data, pos+1);
    } else
        p += Data;
    data = 1;
  } 
}

 END {
    close(InFile);
    print "c" Color1;
    if ( !CG ) {
        while ((getline < InFile) > 0) {
            if ( $1 == "#" || $1 ~ /^xg/ ) continue;
            printf("m%.3f %.3f\n",$1,   $2);
            printf("d%.3f %.3f\n",$1+GDX,$2);
            printf("d%.3f %.3f\n",$1+GDX,$2+GDY);
            printf("d%.3f %.3f\n",$1,   $2+GDY);
            printf("d%.3f %.3f\n",$1,   $2);
            }
        } else {
            for (i=xmin; i <= xmax+1.5*GDX; i+=GDX) {
                printf("m%.3f %.3f\n",i,ymin);
                printf("d%.3f %.3f\n",i,ymax+GDY);
                }
            for (i=ymin; i <= ymax+1.5*GDY; i+=GDY) {
                printf("m%.3f %.3f\n",xmin,i);
                printf("d%.3f %.3f\n",xmax+GDX,i);
                }
            }
    if ( LG >= 1 )
        Legend();
    print "u0";
    print "s8";
    print "c1";
    printf("t905 415 GDX : %10.1f\n",GDX);
    printf("t905 400 GDY : %10.1f\n",GDY);
    printf("t905 385 MIN : %10.1f\n",wmin);
    printf("t905 370 MAX : %10.1f\n",wmax);
    if ( RP == 2 )
        printf("t905 355 SCALE: %9.1f\n",Scale);
    printf("Input-file    : %-12s (%4d lines read)\n", FILENAME, NR) >> MsgOutput;
    }
