# -----------------------------------------------------------------------------
#                       GRTOOL - A Link To Grid Interface
# -----------------------------------------------------------------------------
# FILE    : GRLINKA.AWK  Version 2.5
# AUTHOR  : Heinz & Lukas Spiess, CH-2558 Aegerten
# DATE    : 12 December 1993
# LAST REV: 10 February 1994
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# REMARKS : - grlink1.awk, sorting and grlink2.awk are now integrated in one
#             file and an EXE file can be created with AWKC:
#             awkc -o grlinka.exe -f grlinka.awk
#           - Attention: DOS sort.exe won't work with much data, so it's 
#             recomended to use GNU sort.exe or MKS sort.exe respectively
#             It's assumed that the name is GSORT.EXE.
#           - 3 temporary files gridlink.tmp, gridsort.tmp and grheader.tmp
#             are created during execution (gridlink.tmp is the inputfile for
#             sort.exe, gridsort.tmp is the sorted file and grheader.tmp
#             contains the header line(s) from the input-file). These files
#             are deleted in the batch file or shellscript.
#           - The calling syntax is:
#
#             grlinka GD=grid-distance(s) input-file [> output-file]
#
#             where grlinka is either: awk -f grlinka.awk or grlinka.exe
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#                                    CHANGES   
# 16-12-93: - The calling syntax is now:
#             grlink GD=grid-distance input-file [> output-file]
#
# 17-12-93: - File Header revised. 
#
# 28-01-94: - Filename changed to GRLINKA.AWK
#           - GNU sort.exe renamed to gsort.exe 
#           - New: input header line information (column labels) generated by
#             grmacro.mac (inode jnode xi ...) will be available in the
#             output-file. This file has a header, containing information
#             about the GRTOOL version the file was created, the
#             grid-distance and the column labels. These labels can be used
#             for the dataselection in GRANNOT/GRPARAM. 
# 08-02-94: - GD replaced by GDX and GDY for non qudratic grid squares.
#             For the moment it's still GDX=GDY=GD.
#           - Label line begins no longer with a comment card, but contains
#             the grid extentsions as follows:
#             xg=GDX yg=GDY label1 label2 ... 
# 10-02-94: - The same awk file can used on DOS and UNIX systmes without any
#             changes. The temporary file grtool.tmp, which has to be
#             created by the DOS batch file or by the UNIX shell script
#             allows to specify the message output device by the keyword
#             OUTPUT device and the sort program to be used is referenced by
#             the keyword GRSORT sort-program. Under this is usually 
#             GRSORT gsort and OUTPUT con, under UNIX GRSORT sort and
#             OUTPUT grtool.log or /dev/tty.
# 10-04-20: - New variables GOX/GOY added to control grid cell offset
# ----------------------------------------------------------------------------
function grlink2() {
    xx      = "xx";
    yy      = "yy";
    flag    =    1;

    while ((getline < "gridsort.tmp") > 0) {
    if ((xx!=$1) || (yy!=$2)) {
        if (!flag) {
            printf("%8.3f %8.3f ",xx,yy) ;
            for (i=4;i<=lastNF;i++) printf("%8.3f ",Wert[i]) ;
            printf("\n") ;
            }
        xx=$1;
        yy=$2;
        for (i=4;i<=NF;i++) Wert[i]=$3*$i;
        lastNF=NF;
        } else {
            for (i=4;i<=NF;i++) Wert[i]+=$3*$i;
            }
        flag=0;
        } 
    printf("%8.3f %8.3f ",xx,yy) ;
    for (i=4;i<=lastNF;i++) printf("%8.3f ",Wert[i]) ;
    printf("\n") ;
    }

BEGIN { VER = "2.5";
        getline < "grtool.tmp"
        if ( $1 == "GRLINK2" ){
            grlink2();
            exit;
        }
        printf("# Created by GRLINK (%s)\n", VER);
      }
 
NR == 1 {
    GOX = GOY = 0;
    if ( index(GD,"/") ) {
      split(GD,a,"/");
      if ( a[1] )
        GDX = a[1]; 
      else
        GDX = 1;
      if ( a[2] )
        GDY = a[2];
      else
        GDY = GDX;
      if ( a[3] ) GOX = a[3];
      if ( a[4] ) GOY = a[4];
    } else
      GDX = GDY = GD;
    }

$1 ~ /^[A-Za-z][A-Za-z0-9]*$/ {
    # Write Header
    if( $1 == "inode"){
        printf("%8s %8s ","xg="GDX,"yg="GDY);
        n = split($0,label);
        for ( i=8; i<=n; i++) {
          printf("%8s ",label[i]);
          }
        printf("\n");
        }
    next;
    }

{ 
    xi=$3-GOX ; yi=$4-GOY ; xj=$5-GOX ; yj=$6-GOY ;
    if ((width=$7/2)==0) next;
    werte="";
    for (i=8;i<=NF;i++) werte=werte $i " ";
    ex=xj-xi;
    ey=yj-yi;
    l =sqrt(ex*ex+ey*ey);
    if (l!=0) {
        ex/=l;
        ey/=l;
        }
    nx1= ey*width;
    ny1=-ex*width;
    x[1,1]=nx1+xi; y[1,1]=ny1+yi;
    x[1,2]=nx1+xj; y[1,2]=ny1+yj;
    x[1,3]=xj-nx1; y[1,3]=yj-ny1;
    x[1,4]=xi-nx1; y[1,4]=yi-ny1;
    xmin=x[1,1];ymin=y[1,1]
    xmax=x[1,1];ymax=y[1,1]
    for (i=2;i<=4;i++) {
        xx=x[1,i];
        if (xx<xmin) xmin=xx;
        if (xx>xmax) xmax=xx;
        yy=y[1,i];
        if (yy<ymin) ymin=yy;
        if (yy>ymax) ymax=yy;
        }
    incx=xmin<0 ? (int(xmin/GDX)-1)*GDX : 0;
    incy=ymin<0 ? (int(ymin/GDY)-1)*GDY : 0;
    if (incx) for (i=1;i<=4;i++) x[1,i]=x[1,i]-incx;
    if (incy) for (i=1;i<=4;i++) y[1,i]=y[1,i]-incy;
    x[1,"<"]=xmin-incx
    y[1,"<"]=ymin-incy
    x[1,">"]=xmax-incx
    y[1,">"]=ymax-incy
    a[1]=4;
    ftot2=l*$7*2;
    incx+=GOX;
    incy+=GOY;
    GridVer(1);
}

function GridVer(p) {
    xmin=x[p,"<"];
    xmax=x[p,">"];
    vl1=int(xmin/GDX);
    vl2=int(xmax/GDX);
    if (vl1!=vl2) {
        while ((vl1+1)*GDX<=xmin) { vl1++ }
        while (vl2*GDX<xmax) { vl2++ }
        if ((vl2-vl1)>1) {
             xs=int((vl1+vl2)/2)*GDX;
             Teilen(p,y,x,xs);
             GridVer(p+2);
             GridVer(p+3);
             } else { GridHor(p) }
        } else { GridHor(p) }
    }

function GridHor(p) {
    ymin=y[p,"<"]
    ymax=y[p,">"]
    hl1=int(ymin/GDY);
    hl2=int(ymax/GDY);
    if (hl1!=hl2) {
        while ((hl1+1)*GDY<=ymin) { hl1++ }
        while (hl2*GDY<ymax) { hl2++ }
        if (hl2-hl1>1) {
             ys=int((hl1+hl2)/2)*GDY;
             Teilen(p,x,y,ys);
             GridHor(p+2);
             GridHor(p+3);
             } else { Flaeche(p) }
        } else { Flaeche(p) }
    }

function Teilen(p, x, y, ys) {
    g=p+2; u=1;
    h=p+3; v=1;
    n=a[p];
    yb=y[p,n];
    xb=x[p,n];
    y[g,"<"]=y[p,"<"];
    y[g,">"]=ys;
    y[h,"<"]=ys;
    y[h,">"]=y[p,">"];
    xming=xminh=1e20;
    xmaxg=xmaxh=-1e20;
    for (i=1;i<=n;i++) {
        ya=yb; yb=y[p,i];
        xa=xb; xb=x[p,i];
        if (ya<=ys) {
            x[g,u]=xa;
            y[g,u++]=ya;
	    if (xa<xming) xming=xa;
	    if (xa>xmaxg) xmaxg=xa;
            }
        if (ya>=ys) {
            x[h,v]=xa;
            y[h,v++]=ya;
	    if (xa<xminh) xminh=xa;
	    if (xa>xmaxh) xmaxh=xa;
            }
        if ((ya<ys && yb>ys) || (ya>ys && yb<ys)) {
            if ((d=yb-ya)!=0) {
                m=(xb-xa)/d;
                q=xa-m*ya;
                xs=m*ys+q;
                x[g,u]=x[h,v]=xs;
                y[g,u++]=y[h,v++]=ys;
	        if (xs<xming) xming=xs;
	        if (xs>xmaxg) xmaxg=xs;
	        if (xs<xminh) xminh=xs;
	        if (xs>xmaxh) xmaxh=xs;
                }
            } 
	}
    a[g]=u-1;
    a[h]=v-1;
    x[g,"<"]=xming;
    x[g,">"]=xmaxg;
    x[h,"<"]=xminh;
    x[h,">"]=xmaxh;
   }

function Flaeche(p) {
    f=0;
    n=a[p]
    y2=y[p,n];
    x2=x[p,n];
    for (t=1;t<=n;t++) {
        x1=x2; x2=x[p,t];
        y1=y2; y2=y[p,t];
        f+=x1*y2-x2*y1;
        }
    printf("%7.3f %7.3f  %f  %s\n",vl1*GDX+incx,hl1*GDY+incy,f/ftot2,werte) > "gridlink.tmp";
    }

