# -----------------------------------------------------------------------------
#                       GRTOOL - A Link To Grid Interface
# -----------------------------------------------------------------------------
# FILE    : GRPARAMA.AWK  Version 2.5
# AUTHOR  : Lukas Spiess, CH-2558 Aegerten
# DATE    :  6 November 1992
# LAST REV: 10 February 1994
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# REMARKS : - This utility is used to creat parameter-files. To creat an 
#             annotation-file you must specify some parameters such as
#             grid-distances, colors, type of representation, type of legen 
#             (nonw/default/customized) and so on.
#           - It is possible to specify an existing parameter-file for
#             default values.
#           - This file can be converted to an .EXE file using AWKC:
#             awkc -o grparama.exe -f grparama.awk
#           - The calling syntax is simply:
#
#             grparama
#
#             where grparama is either: awk -f grparama.awk or grparama.exe
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#                                    CHANGES   
#
# 06-11-92: - little syntax changes in function YesNo
# 09-12-93: - Outfile handling with variable out (default filename= paramet.par)
#             instead of ARGC in order to function properly after the
#             compilation with AWKC. The calling syntax is now:
#             grparam [out=filename] (this filename is referred to as the 
#                                     parameter-file in the annotation utility)
# 09-17-93: - File header revised. 
# 27-01-94: - Filename changed to GRPARAMA.AWK
# 31-01-94: - New default value handling: You can specify an existing parameter-
#             file as a default value "server". So all values specified in this
#             "reference" parameter-file can be taken over to the new one by
#             simple typing ENTER at the prompts.
#             If no reference parameter is specified, arbitrary default values
#             are available. The reference parameter-file must be named
#             explicitly as DEFAULT.PAR.
#           - The output-parameter-file can no longer be specified by 
#             commandline. A apropriate question asks for the filename.
#             Default name "paramet.par" is still available by typing ENTER.
#             Thus the calling syntax is simple:
#             grparam [and DEFAULT.PAR can serve as reference parameter-file]
# 08-02-94: - New Question "Name of existing parameter-file for default values
#             (optional)", DEFAULT.PAR is no longer supported. 
#           - Default-filename paramet.par is no longer supported.
#           - LogFile changed to permanent Logfile grtool.log 
#           - Grid-distance handling changed to GDX and GDY conventions
#           - Class specification (keyword LC) modified as follows
#             old: LC  nr  from  to  -c color  -h flag  dy  -v flag  dx  text
#             new: LC  nr  from  to     color           dy           dx  text
#             e.g. LC  1   0     100    4               .2           .2  '-100'
# 10-02-94: - The same awk file can used on DOS and UNIX systmes without any
#             changes. The temporary file grtool.tmp, which has to be
#             created by the DOS batch file or by the UNIX shell script
#             allows to specify the message output device by the keyword
#             OUTPUT device and the sort program to be used is referenced by
#             the keyword GRSORT sort-program. Under this is usually 
#             GRSORT gsort and OUTPUT con, under UNIX GRSORT sort and
#             OUTPUT grtool.log or /dev/tty.
# -----------------------------------------------------------------------------
 
BEGIN {
    LogFile   = "grtool.log";
    while ((getline < "grtool.tmp") > 0)
      if ( $1 == "OUTPUT" ) LogFile=$2;

    ParFile    = "";
    defParFile = " ";

    cdef[0]=lodef[0]=hidef[0]=Hordef[0]=Verdef[0]=cdydef[0]=cdxdef[0]=0;
    Textdef[0]="";
    RPdef=LGdef=CGdef=TSdef=TIdef=NCdef=0;
    GDXdef=DNdef=1;
    GDYdef=0;
    GCdef=PCdef=4;
    TIdef=GDXdef;
    TSdef=GDXdef / 2;
    LWdef=LHdef=NSdef=DTdef=DFdef=NXdef=NYdef=LTdef="*";
    xldef=yldef=xhdef=yhdef=xdispdef=ydispdef=txdef=tydef="*";

    printf("\nEnter: Name of existing parameter-file for default values (optional) = ");
    input = getline;
    if ( $0 != "" )
      defParFile = $0;

    while ((getline < defParFile) > 0) {
        l++;
        if ( $1 == "#") continue;
        if ( $1 == "CG" ) { CGdef    = $2; continue; }
        if ( $1 == "RP" ) { RPdef    = $2; continue; }
        if ( $1 == "LG" ) { LGdef    = $2; continue; }
        if ( $1 == "TS" ) { TSdef    = $2; continue; }
        if ( $1 == "TI" ) { TIdef    = $2; continue; }
        if ( $1 == "NC" ) { NCdef    = $2; continue; }
        if ( $1 == "DT" ) { DTdef    = $2; continue; }
        if ( $1 == "DF" ) { DFdef    = $2; continue; }
        if ( $1 == "DN" ) { DNdef    = $2; continue; }
        if ( $1 == "SC" ) { SCdef    = $2; continue; }
        if ( $1 == "GC" ) { GCdef    = $2; continue; }
        if ( $1 == "PC" ) { PCdef    = $2; continue; }
        if ( $1 == "ND" ) { NDdef    = $2; continue; }
        if ( $1 == "NS" ) { NSdef    = $2; continue; }
        if ( $1 == "GD" ) { GDXdef   = $2; GDYdef  = $3; continue; }
        if ( $1 == "DG" ) { NXdef    = $2; NYdef   = $3; continue; }
        if ( $1 == "LK" ) { xldef    = $2; yldef   = $3; continue; }
        if ( $1 == "TK" ) { xhdef    = $2; yhdef   = $3; continue; }
        if ( $1 == "TD" ) { txdef    = $2; tydef   = $3; continue; }
        if ( $1 == "LB" ) { LWdef    = $2; LHdef   = $3; continue; }
        if ( $1 == "LD" ) { xdispdef = $2; ydispdef= $3; continue; }
        if ( $1 == "LT" ) { 
            split($0,array,"'");
            LTdef = array[2]; 
            continue;
	    }
        if ( $1 == "LC" ) {
            i         = $2;
            lodef[i]  = $3; hidef[i]  = $4;
	    cdef[i]   = $5;
	    Hordef[i] = Verdef[i] = 0;
            if ( $6 ) {
              cdydef[i] = $6;
	      Hordef[i] = 1;
              }
            if ( $7 ) {
              cdxdef[i] = $7;
              Verdef[i] = 1;
              }
            split($0,array,"'");
            Textdef[i]= array[2]; 
            continue;
            }
        printf("FILE %s, LINE %d NOT RECOGNIZED:\n%s\n",ParFile,l,$0) >> LogFile;
        }

  do {
    printf("\nEnter: Name of parameter-file to be generated = ");
    input = getline;
    if ( $0 != "" )
      ParFile = $0;
  } while (!ParFile)

  Enter2("Horizontal,vertical grid distance", "GD", GDXdef, GDYdef);
  YesNo("Draw complete grid", "CG", CGdef);
  Enter1("Grid color", "GC", GCdef);
  Enter1("Name or number of data item to be displayed", "DN", DNdef);
  Selection("Type of gird representation", "Classes defined by value interval", "Numeric values", "Hatching with proportional density", "RP", RPdef);
  if ( RP == 1 ) {
      Enter1("Color of the numbers", "PC", PCdef);
      Enter1("Text size of the numbers", "NS", NSdef);
      Enter1("Minimal field width", "DT", DTdef);
      Enter1("Number of digits after decimal point", "DF", DFdef);
      Enter2("Offset of the number inside grid box (xdisp, ydisp)", "DG",NXdef,NYdef);
        }
  if ( RP == 2 ) {
    Enter1("Scale factor","SC",SCdef);
    Enter1("Color of pattern","PC",PCdef);
    }
  if ( RP == 0 )
    Selection("Type of legend","None","Default","Customized","LG", LGdef);
  if ( LG && RP == 0 ) {
    printf("\nEnter: Text for legend title (%s) = ", LTdef);
    input = getline;
    if ( (Title = $0) == "" ) Title = LTdef;
    printf("LT '%s'   Legend title\n", Title) > ParFile;
    }
  if ( LG == 1 && RP == 0 )
    Enter2("Lower left corner of first legend box (xl, yl)","LK",xldef,yldef);
  if ( LG == 2 && RP == 0 ) {
    Enter1("Size of the legend title", "TI", TIdef);
    Enter2("Offset for legend title (xdisp, ydisp)", "TK", xhdef, yhdef);
    Enter1("Size of the legend text", "TS", TSdef);
    Enter2("Lower left corner of first legend box (xl, yl)", "LK", xldef,yldef);
    Enter2("Width and height of legend box (width, height)", "LB", LWdef,LHdef);
    Enter2("Offset for next legend box (xdisp, ydisp)", "LD",xdispdef,ydispdef);
    Enter2("Offset for legend text (xdisp, ydisp)", "TD",txdef, tydef);
    }
  if ( RP == 0 ) {
    Enter1("Number of classes", "NC", NCdef);
    for (i=1; i <= NC; i++) {
      do {
        if (i<=NCdef)
          k=i;  
        else  
          k=0;
        ok = 1;
        printf("\nEnter: Class interval %2d: (from, to) (%s, %s) =", i,lodef[k], hidef[k]);
        input = getline;
        if ( $0 != "" ) {
          if ( index($0,",") !=  0 )
            sep = ",";
          else if ( index($0," ") !=  0 )
            sep = " ";
          else ok = 0;
            split($0,wert,sep);
          } else
            wert[1]=wert[2]="";
        if ( i <= NCdef ) {
          wert[1] == "" ? lo = lodef[k] : lo = wert[1];
          wert[2] == "" ? hi = hidef[k] : hi = wert[2];
        } else {
          wert[1] == "" ? ok = 0 : lo = wert[1];
          wert[2] == "" ? ok = 0 : hi = wert[2];
          }
      } while (!ok)
      printf("\nEnter: Color for class %2d (%s) = ", i, cdef[k]);
      input = getline;
      if ( $1 == "" )
        CO = cdef[k];
      else
        CO = $1;
      do {
        ok = 1; Hor = 0; Ver = 0; Def=3;
        if      (  Hordef[k] && !Verdef[k] ) Def=1;
        else if ( !Hordef[k] &&  Verdef[k] ) Def=2;
        else if (  Hordef[k] &&  Verdef[k] ) Def=3;
        printf("\nSelect: Hatching pattern for class %2d\n", i);
        printf("        1= Horizontal lines\n");
        printf("        2= Vertical lines\n");
        printf("        3= Horizontal and vertical lines\n");
        printf("       (%s) ",Def);
        input = getline;
        if ( $1 == "") $1=Def
        if ( $1 == 1 ) 
          Hor = 1;
        else if ( $1 == 2 ) 
          Ver = 1;
        else if ( $1 == 3 ) 
          Hor = Ver = 1;
        else {
          printf("A number between 1 and 3 or ENTER must be entered !\n");
          ok = 0;
          }
      } while (!ok)
      dy = 0;
      if ( Hor == 1 ) {
        printf("\nEnter: Distance between 2 horizontal lines (%s) = ", cdydef[k]);
        input = getline;
        if ( $0 == "")
          dy = cdydef[k];
        else
          dy = $1;
        }
      dx = 0;
      if ( Ver == 1) {
        printf("\nEnter: Distance between 2 vertical lines (%s) = ", cdxdef[k]);
        input = getline;
        if ( $0 == "")
          dx = cdxdef[k];
        else
          dx = $1;
        }
      do {
        ok = 1;
        printf("\nEnter: Text for class %2d (%s) = ", i, Textdef[k]);
        input = getline
        if ( $0 != "" ) 
          T = $0;
        else if ( i <= NCdef )
          T = Textdef[k];
        else
          ok = 0;
      } while ( !ok )
      printf("LC %2d %9s %9s  %2d  %5s  %5s '%s'\n",i,lo,hi,CO,dy,dx,T) > ParFile;
      }
    }
  print "Parameter-file= " ParFile >> LogFile;
  }

function YesNo(Question,Keyword,Def) {
    do {
        Val ="y";
        if (!Def) Val="n" ;
        ok     = 1;
        Answer = 0;
        printf("\n%s (%s) ? ", Question, Val);
        input  = getline;
        if ( $0 == "" ) {
            Answer = Def;
        } else if ( $0 == "y" || $0 == "Y" ) {
            Answer = 1;
            } else if ( !($0 == "n") && !($0 == "N") )  { 
                printf("Answer must be: Yes or No !\n"); 
                ok = 0;
                }
        } while (!ok);
    printf("%s %10s            %-30s\n",Keyword,Answer,Question) > ParFile;
    }

function Enter1(Question,Keyword,Def) {
    printf("\nEnter: %s (%s) = ",Question,Def);
    input = getline;
    Val   = Def;
    if ( $0 != "" )
        Val = $0;
    printf("%s %10s            %-30s\n",Keyword,Val,Question) > ParFile;
    if (Keyword == "NC") { NC = Val }
    }

function Enter2(Question,Keyword,Def1,Def2) {
    do {
        ok = 1;
        printf("\nEnter: %s (%s,%s) = ",Question,Def1,Def2);
        input = getline;
        sep = "";
        if ( $0 != "" ) {
            if ( index($0,",") !=  0 )
              sep = ",";
            else if ( index($0," ") !=  0 )
              sep = " ";
            else if ( Keyword != "GD" )
              ok = 0;
            if ( sep )
              split($0,Wert,sep);
            else if ( Keyword == "GD" )
              Wert[1] = Wert[2] = $1;
         } else 
            Wert[1] = Wert[2] = "";
        Wert[1] == "" ? W1 = Def1 : W1 = Wert[1];
        Wert[2] == "" ? W2 = Def2 : W2 = Wert[2];
        } while (!ok);
    printf("%s %10s %10s %-30s\n",Keyword,W1,W2,Question) > ParFile;
    if ( Keyword == "GD" ) {
	if(Wert[3]+0 || Wert[4]+0)
        printf("%s %10s %10s %-30s\n","GO",Wert[3]+0,Wert[4]+0,"horizontal, vertical grid offset") > ParFile;
        }
    }

function Selection(Title, Item1, Item2, Item3, Keyword, Def) {
    do {
        ok = 1;
        printf("\nSelect: %s\n",Title);
        printf("        1= %s\n",Item1);
        printf("        2= %s\n",Item2);
        printf("        3= %s\n",Item3);
        printf("       (%s) ",Def+1);
        input = getline;
        if ( $1 == 1 ) 
            Select = 0;
        else if ( $1 == 2 ) 
            Select = 1;
        else if ( $1 == 3 ) 
            Select = 2;
        else if ( $0 == "") 
            Select = Def;
        else {
            printf("A number between 1 and 3 or ENTER must be entered !\n");
            ok = 0;
            }
        } while (!ok)
    if ( Keyword == "RP" ) {
        RP = Select;
        printf("RP %10s            Representation (0=Class,1=Numbers,2=Proportional)\n",RP) > ParFile;
        }
    if ( Keyword == "LG" ) {
        LG = Select;
        printf("LG %10s            Legend (0=None,1=Default,2=Customized)\n",LG) > ParFile;
        }
    }
